@@ -12,7 +12,9 @@ from hypercorn.typing import HTTPScope, Scope
 
 
 @pytest.mark.asyncio
-async def test_spawn_app(event_loop: asyncio.AbstractEventLoop, http_scope: HTTPScope) -> None:
+async def test_spawn_app(http_scope: HTTPScope) -> None:
+    event_loop: asyncio.AbstractEventLoop = asyncio.get_running_loop()
+
     async def _echo_app(scope: Scope, receive: Callable, send: Callable) -> None:
         while True:
             message = await receive()
@@ -31,9 +33,9 @@ async def test_spawn_app(event_loop: asyncio.AbstractEventLoop, http_scope: HTTP
 
 
 @pytest.mark.asyncio
-async def test_spawn_app_error(
-    event_loop: asyncio.AbstractEventLoop, http_scope: HTTPScope
-) -> None:
+async def test_spawn_app_error(http_scope: HTTPScope) -> None:
+    event_loop: asyncio.AbstractEventLoop = asyncio.get_running_loop()
+
     async def _error_app(scope: Scope, receive: Callable, send: Callable) -> None:
         raise Exception()
 
@@ -41,17 +43,3 @@ async def test_spawn_app_error(
     async with TaskGroup(event_loop) as task_group:
         await task_group.spawn_app(ASGIWrapper(_error_app), Config(), http_scope, app_queue.put)
     assert (await app_queue.get()) is None
-
-
-@pytest.mark.asyncio
-async def test_spawn_app_cancelled(
-    event_loop: asyncio.AbstractEventLoop, http_scope: HTTPScope
-) -> None:
-    async def _error_app(scope: Scope, receive: Callable, send: Callable) -> None:
-        raise asyncio.CancelledError()
-
-    app_queue: asyncio.Queue = asyncio.Queue()
-    with pytest.raises(asyncio.CancelledError):
-        async with TaskGroup(event_loop) as task_group:
-            await task_group.spawn_app(ASGIWrapper(_error_app), Config(), http_scope, app_queue.put)
-    assert (await app_queue.get()) is None
