@@ -13,12 +13,15 @@ from ..helpers import echo_framework
 
 
 @pytest.mark.asyncio
-async def test_completes_on_closed(event_loop: asyncio.AbstractEventLoop) -> None:
+async def test_completes_on_closed() -> None:
+    event_loop: asyncio.AbstractEventLoop = asyncio.get_running_loop()
+
     server = TCPServer(
         ASGIWrapper(echo_framework),
         event_loop,
         Config(),
         WorkerContext(None),
+        {},
         MemoryReader(),  # type: ignore
         MemoryWriter(),  # type: ignore
     )
@@ -29,12 +32,15 @@ async def test_completes_on_closed(event_loop: asyncio.AbstractEventLoop) -> Non
 
 
 @pytest.mark.asyncio
-async def test_complets_on_half_close(event_loop: asyncio.AbstractEventLoop) -> None:
+async def test_complets_on_half_close() -> None:
+    event_loop: asyncio.AbstractEventLoop = asyncio.get_running_loop()
+
     server = TCPServer(
         ASGIWrapper(echo_framework),
         event_loop,
         Config(),
         WorkerContext(None),
+        {},
         MemoryReader(),  # type: ignore
         MemoryWriter(),  # type: ignore
     )
@@ -45,5 +51,5 @@ async def test_complets_on_half_close(event_loop: asyncio.AbstractEventLoop) ->
     data = await server.writer.receive()  # type: ignore
     assert (
         data
-        == b"HTTP/1.1 200 \r\ncontent-length: 335\r\ndate: Thu, 01 Jan 1970 01:23:20 GMT\r\nserver: hypercorn-h11\r\n\r\n"  # noqa: E501
+        == b"HTTP/1.1 200 \r\ncontent-length: 348\r\ndate: Thu, 01 Jan 1970 01:23:20 GMT\r\nserver: hypercorn-h11\r\n\r\n"  # noqa: E501
     )
