@@ -21,6 +21,7 @@ from hypercorn.protocol.ws_stream import (
     WSStream,
 )
 from hypercorn.typing import (
+    ConnectionState,
     WebsocketAcceptEvent,
     WebsocketCloseEvent,
     WebsocketResponseBodyEvent,
@@ -182,6 +183,7 @@ async def test_handle_request(stream: WSStream) -> None:
             headers=[(b"sec-websocket-version", b"13")],
             raw_path=b"/?a=b",
             method="GET",
+            state=ConnectionState({}),
         )
     )
     stream.task_group.spawn_app.assert_called()  # type: ignore
@@ -200,9 +202,40 @@ async def test_handle_request(stream: WSStream) -> None:
         "server": None,
         "subprotocols": [],
         "extensions": {"websocket.http.response": {}},
+        "state": ConnectionState({}),
     }
 
 
+@pytest.mark.asyncio
+async def test_handle_data_before_acceptance(stream: WSStream) -> None:
+    await stream.handle(
+        Request(
+            stream_id=1,
+            http_version="2",
+            headers=[(b"sec-websocket-version", b"13")],
+            raw_path=b"/?a=b",
+            method="GET",
+            state=ConnectionState({}),
+        )
+    )
+    await stream.handle(
+        Data(
+            stream_id=1,
+            data=b"X",
+        )
+    )
+    assert stream.send.call_args_list == [  # type: ignore
+        call(
+            Response(
+                stream_id=1,
+                headers=[(b"content-length", b"0"), (b"connection", b"close")],
+                status_code=400,
+            )
+        ),
+        call(EndBody(stream_id=1)),
+    ]
+
+
 @pytest.mark.asyncio
 async def test_handle_connection(stream: WSStream) -> None:
     await stream.handle(
@@ -212,6 +245,7 @@ async def test_handle_connection(stream: WSStream) -> None:
             headers=[(b"sec-websocket-version", b"13")],
             raw_path=b"/?a=b",
             method="GET",
+            state=ConnectionState({}),
         )
     )
     await stream.app_send(cast(WebsocketAcceptEvent, {"type": "websocket.accept"}))
@@ -241,6 +275,7 @@ async def test_send_accept(stream: WSStream) -> None:
             headers=[(b"sec-websocket-version", b"13")],
             raw_path=b"/",
             method="GET",
+            state=ConnectionState({}),
         )
     )
     await stream.app_send(cast(WebsocketAcceptEvent, {"type": "websocket.accept"}))
@@ -260,6 +295,7 @@ async def test_send_accept_with_additional_headers(stream: WSStream) -> None:
             headers=[(b"sec-websocket-version", b"13")],
             raw_path=b"/",
             method="GET",
+            state=ConnectionState({}),
         )
     )
     await stream.app_send(
@@ -284,6 +320,7 @@ async def test_send_reject(stream: WSStream) -> None:
             headers=[(b"sec-websocket-version", b"13")],
             raw_path=b"/",
             method="GET",
+            state=ConnectionState({}),
         )
     )
     await stream.app_send(
@@ -318,6 +355,7 @@ async def test_invalid_server_name(stream: WSStream) -> None:
             headers=[(b"host", b"example.com"), (b"sec-websocket-version", b"13")],
             raw_path=b"/",
             method="GET",
+            state=ConnectionState({}),
         )
     )
     assert stream.send.call_args_list == [  # type: ignore
@@ -343,6 +381,7 @@ async def test_send_app_error_handshake(stream: WSStream) -> None:
             headers=[(b"sec-websocket-version", b"13")],
             raw_path=b"/",
             method="GET",
+            state=ConnectionState({}),
         )
     )
     await stream.app_send(None)
@@ -370,6 +409,7 @@ async def test_send_app_error_connected(stream: WSStream) -> None:
             headers=[(b"sec-websocket-version", b"13")],
             raw_path=b"/",
             method="GET",
+            state=ConnectionState({}),
         )
     )
     await stream.app_send(cast(WebsocketAcceptEvent, {"type": "websocket.accept"}))
@@ -392,6 +432,7 @@ async def test_send_connection(stream: WSStream) -> None:
             headers=[(b"sec-websocket-version", b"13")],
             raw_path=b"/",
             method="GET",
+            state=ConnectionState({}),
         )
     )
     await stream.app_send(cast(WebsocketAcceptEvent, {"type": "websocket.accept"}))
@@ -407,7 +448,9 @@ async def test_send_connection(stream: WSStream) -> None:
 
 
 @pytest.mark.asyncio
-async def test_pings(stream: WSStream, event_loop: asyncio.AbstractEventLoop) -> None:
+async def test_pings(stream: WSStream) -> None:
+    event_loop: asyncio.AbstractEventLoop = asyncio.get_running_loop()
+
     stream.config.websocket_ping_interval = 0.1
     await stream.handle(
         Request(
@@ -416,6 +459,7 @@ async def test_pings(stream: WSStream, event_loop: asyncio.AbstractEventLoop) ->
             headers=[(b"sec-websocket-version", b"13")],
             raw_path=b"/?a=b",
             method="GET",
+            state=ConnectionState({}),
         )
     )
     async with TaskGroup(event_loop) as task_group:
